"""
Testing suite for aat_wrest
"""

import argparse
import logging
import sys
from unittest.mock import MagicMock, patch

import pendulum as pendulum
from aat_wrest.metadata_wrester import WrestWorkflowMetadata

logger = logging.getLogger("test_aat_wrest")
logger.setLevel(logging.INFO)
logger.addHandler(logging.StreamHandler(sys.stdout))


args_cal = argparse.Namespace(
    stdcals="uid://evla/execblock/125dac14-c53b-4489-9c43-7d6e2739ec32", stdimg=None
)
result_cal = (
    '{"sdmId": "17B-197.sb34663512.eb34806505.58108.78427738426", "projectCode": "17B-197", '
    '"title": "Radio-Active CoWS: Extended FIRST Sources in MaDCoWS Clusters", '
    '"startTime": 58108.7842916667, "observer": "Anthony Gonzalez", "telescope": "EVLA",'
    ' "created_at": 2021-01-01T00:00:00}'
)

args_image = argparse.Namespace(stdcals=None, stdimg="brain_000.58099.67095825232")
result_img = (
    '{"product_locator": "uid://evla/execblock/ec082e65-452d-4fec-ad88-f5b4af1f9e36",'
    ' "projectCode": "Operations", "title": "", "startTime": 58099.6710792824, '
    '"observer": "VLA Operations", "telescope": "EVLA", "created_at": "2021-06-30T20:18:21"}'
)

args_carta = argparse.Namespace(
    carta="uid://evla/image/3d3db489-9331-4e61-aa80-002bc2989b1e", stdcals=None, stdimg=None
)
result_carta = '{"imageName": "VLASS1.1.ql.T29t04.J094850+743000.10.2048.v5"}'


def mock_wrester(args: argparse.Namespace) -> WrestWorkflowMetadata:
    with patch("psycopg2.connect") as mock_connect:
        if args.stdcals is not None:
            return WrestWorkflowMetadata(connection=mock_connect, spl=args.stdcals)
        if args.stdimg is not None:
            return WrestWorkflowMetadata(connection=mock_connect, sdm_id=args.stdimg)
        if args.carta is not None:
            return WrestWorkflowMetadata(connection=mock_connect, spl=args.carta)


cal_wrester = mock_wrester(args_cal)
img_wrester = mock_wrester(args_image)
carta_wrester = mock_wrester(args_carta)


class TestAatWrest:
    @patch("json.dumps", MagicMock(return_value=result_cal))
    def test_wrest_standard_cal_info(self):
        cal_wrester.conn.cursor.return_value.fetchall.return_value = [
            "17B-197.sb34663512.eb34806505.58108.78427738426",
            "17B-197",
            "Radio-Active CoWS: Extended FIRST Sources in MaDCoWS Clusters",
            58108.7842916667,
            "Anthony Gonzalez",
            "EVLA",
            str(pendulum.now().in_timezone("UTC").format("YYYY-MM-DDTHH:mm:ss")),
        ]
        assert args_cal.stdcals == "uid://evla/execblock/125dac14-c53b-4489-9c43-7d6e2739ec32"
        value = cal_wrester.wrest_standard_cal_info()
        assert (
            value
            == '{"sdmId": "17B-197.sb34663512.eb34806505.58108.78427738426", "projectCode": "17B-197", '
            '"title": "Radio-Active CoWS: Extended FIRST Sources in MaDCoWS Clusters", '
            '"startTime": 58108.7842916667, "observer": "Anthony Gonzalez", "telescope": "EVLA",'
            ' "created_at": 2021-01-01T00:00:00}'
        )

    @patch("json.dumps", MagicMock(return_value=result_img))
    def test_wrest_standard_image_info(self):
        img_wrester.conn.cursor.return_value.fetchall.return_value = [
            "uid://evla/execblock/ec082e65-452d-4fec-ad88-f5b4af1f9e36",
            "Operations",
            "",
            58099.6710792824,
            "VLA Operations",
            "EVLA",
            "2021-06-30T20:18:21",
        ]
        assert args_image.stdimg == "brain_000.58099.67095825232"
        value = img_wrester.wrest_standard_image_info()
        assert (
            value
            == '{"product_locator": "uid://evla/execblock/ec082e65-452d-4fec-ad88-f5b4af1f9e36",'
            ' "projectCode": "Operations", "title": "", "startTime": 58099.6710792824, '
            '"observer": "VLA Operations", "telescope": "EVLA", "created_at": "2021-06-30T20:18:21"}'
        )

    @patch("json.dumps", MagicMock(return_value=result_carta))
    def test_wrest_image_name(self):
        carta_wrester.conn.cursor.return_value.fetchall.return_value = [
            "VLASS1.1.ql.T29t04.J094850+743000.10.2048.v5"
        ]
        assert args_carta.carta == "uid://evla/image/3d3db489-9331-4e61-aa80-002bc2989b1e"
        value = carta_wrester.wrest_image_name()
        assert value == '{"imageName": "VLASS1.1.ql.T29t04.J094850+743000.10.2048.v5"}'
